@@ -6,9 +6,9 @@ from collections import OrderedDict, defaultdict
 from collections.abc import Sequence
 import pkg_resources
 import json
-import sys
+from math import isnan
 # import faster_than_csv as csv
-
+import csv
 import pandas as pd
 from dataclasses import dataclass
 from typing import DefaultDict, Dict, List, Optional, Tuple, Union, Generator, Any
@@ -23,7 +23,8 @@ except ModuleNotFoundError or ImportError:
 _RESOURCE_PACKAGE = __name__
 
 HOME_PATH = '/'.join(('Lexique', ''))
-_RESOURCE_PATH = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'Lexique383/Lexique383.txt')
+_RESOURCE_PATH_csv = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'Lexique383/Lexique383.txt')
+_RESOURCE_PATH_xlsb = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'Lexique383/Lexique383.xlsb')
 _VALUE_ERRORS_PATH = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'errors/value_errors.json')
 _LENGTH_ERRORS_PATH = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'errors/length_errors.json')
 
@@ -95,7 +96,8 @@ class LexItem(LexEntryTypes):
         """
         | Converts the LexItem to a dict containing its attributes and their values
 
-        :return: Dictionary with key/values correspondence wit LexItem objects
+        :return:
+            Dictionary with key/values correspondence wit LexItem objects.
         """
         attributes = []
         for attr in self.__slots__:
@@ -114,10 +116,12 @@ class Lexique383:
     This is the class handling the lexique database.
     It provides methods for interacting with the Lexique DB
     and retrieve lexical items.
-    All the lexical items are then stored in an Ordered Dict called
+    All the lexical items are then stored in an Ordered Dict.
 
     :param lexique_path: string.
-        Path to the lexique csv file.
+        Path to the lexique file.
+    :param parser_type: string.
+        'pandas_csv', 'std_csv', 'csv' and 'xlsb' are valid values.
     """
 
     lexique = OrderedDict()
@@ -125,53 +129,83 @@ class Lexique383:
     length_errors = []
     lemmes = defaultdict(list)
 
-    def __init__(self, lexique_path: Optional[str] = None) -> None:
+    def __init__(self, lexique_path: Optional[str] = None, parser_type: str = 'std_csv') -> None:
         self.lexique_path = lexique_path
+        if parser_type not in {'xlsb', 'pandas_csv', 'csv', 'std_csv'}:
+            raise ValueError(f"The value {parser_type} is not permitted. Only 'pandas_csv', 'std_csv', 'csv' and "
+                             f"'xlsb' are valid values.")
         if lexique_path:
             try:
-                self._parse_lexique(self.lexique_path)
-            except UnicodeDecodeError:
-                sys.exit(0)
-            except FileNotFoundError:
+                self._parse_lexique(self.lexique_path, parser_type)
+            except UnicodeDecodeError as e:
+                raise UnicodeError(f"There was a unicode error while parsing {type(lexique_path)}.") from e
+            except FileNotFoundError as e:
                 if isinstance(lexique_path, str):
-                    raise ValueError(f"Argument 'lexique_path' must be a valid path to Lexique383")
+                    raise ValueError(f"Argument 'lexique_path' must be a valid path to Lexique383") from e
                 if not isinstance(lexique_path, str):
-                    raise TypeError(f"Argument 'lexique_path' must be of type String, not {type(lexique_path)}")
+                    raise TypeError(f"Argument 'lexique_path' must be of type String, not {type(lexique_path)}") from e
         else:
             try:
                 # Tries to load the pre-shipped Lexique38X if no path file to the lexicon is provided.
-                self._parse_lexique(_RESOURCE_PATH)
-            except UnicodeDecodeError:
-                sys.exit(0)
-            except FileNotFoundError:
-                if isinstance(_RESOURCE_PATH, str):
-                    raise ValueError(f"Argument 'lexique_path' must be a valid path to Lexique383")
-                if not isinstance(_RESOURCE_PATH, str):
-                    raise TypeError(f"Argument 'lexique_path'must be of type String, not {type(_RESOURCE_PATH)}")
+                self._parse_lexique(_RESOURCE_PATH_csv, parser_type)
+            except UnicodeDecodeError as e:
+                raise UnicodeError(f"There was a unicode error while parsing {type(_RESOURCE_PATH_csv)}.") from e
+            except FileNotFoundError as e:
+                if isinstance(_RESOURCE_PATH_csv, str):
+                    raise ValueError(f"Argument 'lexique_path' must be a valid path to Lexique383") from e
+                if not isinstance(_RESOURCE_PATH_csv, str):
+                    raise TypeError(f"Argument 'lexique_path'must be of type String, not {type(_RESOURCE_PATH_csv)}") from e
         return
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return '{0}.{1}'.format(__name__, self.__class__.__name__)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.lexique)
 
-    def _parse_lexique(self, lexique_path: str) -> None:
+    @staticmethod
+    def _parse_csv(lexique_path: str) -> Generator[list, Any, None]:
         """
-        | Parses the given lexique file and creates a hdf5 table to store the data.
+
+        :param lexique_path:
+            Path to the lexique file.
+        :return: generator of rows:
+            Content of the Lexique38x database.
+        """
+        with open(lexique_path, 'r', encoding='utf-8', errors='ignore') as csv_file:
+            raw_content = csv_file.readlines()
+            content = (row.strip().split('\t') for row in raw_content[1:])
+            return content
+
+    def _parse_lexique(self, lexique_path: Optional[str], parser_type: str) -> None:
+        """
+        | Parses the given lexique file and creates 2 hash tables to store the data.
 
         :param lexique_path: string.
-            Path to the lexique csv file.
+            Path to the lexique file.
+        :param parser_type: string.
+            Can be either 'csv', 'pandas_csv', 'std_csv' or 'xlsb'.
         :return:
         """
-        # Create a dataframe from csv
         try:
-            df = pd.read_csv(lexique_path, delimiter='\t')
+            if parser_type == 'xlsb':
+                df = pd.read_excel(lexique_path, engine='pyxlsb')
+                content = (list(row) for row in df.values)
+            elif parser_type == 'pandas_csv':
+                df = pd.read_csv(lexique_path, delimiter='\t')
+                content = (list(row) for row in df.values)
+            elif parser_type == 'csv':
+                content = self._parse_csv(lexique_path)
+            elif parser_type == 'std_csv':
+                raw_content = csv.reader(lexique_path, delimiter='\t')
+                raw_content.__next__()
+                content = raw_content
+            else:
+                content = self._parse_csv(lexique_path)
         except UnicodeDecodeError:
-            logger.warn('there was an issue while parsing the file {0}'.format(lexique_path))
-            raise UnicodeDecodeError
-        # Generator comprehension to create a list of lists from Dataframe rows
-        content = (list(row) for row in df.values)
+            logger.warn(f"there was an issue while parsing the file {lexique_path}."
+                        f" Trying again with built-in csv parser")
+            content = self._parse_csv(lexique_path)
         self._create_db(content)
         if self.value_errors:
             self._save_errors(self.value_errors, _VALUE_ERRORS_PATH)
@@ -181,8 +215,8 @@ class Lexique383:
 
     def _create_db(self, lexicon: Generator[list, Any, None]) -> None:
         """
-        | Creates an hash table populated with the entries in lexique if it does not exist yet.
-        | It stores the hash table database for fast access.
+        | Creates 2 hash tables populated with the entries in lexique if it does not exist yet.
+        | One hash table holds the LexItems, the other holds the same data but grouped by lemmma to give access to all lexical forms of a word.
 
         :param lexicon: Iterable.
             Iterable containing the lexique383 entries.
@@ -191,7 +225,7 @@ class Lexique383:
         for row in lexicon:
             try:
                 converted_row_fields = self._convert_entries(row)
-            except ValueError as e:
+            except ValueError:
                 continue
             lexical_entry = LexItem(*converted_row_fields)
             self.lemmes[lexical_entry.lemme].append(lexical_entry)
@@ -210,17 +244,20 @@ class Lexique383:
         | a new list with typed entries.
 
         :param row_fields:
-        :return: converted_row_fields:
+            List of column entries representing a row.
+        :return: typed_row_fields:
+            List of typed column entries representing a typed row.
         """
         errors = defaultdict(list)
         converted_row_fields = []
         for attr, value in zip(LEXIQUE383_FIELD_NAMES, row_fields):
-            if isinstance(value, float):
+            if isinstance(value, float) and isnan(value):
                 value = ''
             if attr in {'freqlemfilms2', 'freqlemlivres', 'freqfilms2', 'freqlivres', 'old20', 'pld20'}:
-                if (value != '' or value != ' ') and ',' in value:
-                    value = value.replace(',', '.')
-                    value = float(value)
+                if not isinstance(value, float):
+                    if (value != '' or value != ' ') and ',' in value:
+                        value = value.replace(',', '.')
+                        value = float(value)
             if attr == 'islem':
                 if isinstance(value, str):
                     value = value.strip()
@@ -249,24 +286,26 @@ class Lexique383:
             row_fields = converted_row_fields
         return row_fields
 
-    def get_lex(self, words: Union[Tuple[str, str, str, str], str]) -> OrderedDict:
+    def get_lex(self, words: Union[Tuple[str], str]) -> OrderedDict:
         """
         Recovers the lexical entries for the words in the sequence
 
         :param words:
-        :return: Dictionary of LexItems:
+            A string or a tuple of multiple strings for getting the LexItems for multiple words.
+        :return:
+            Dictionary of LexItems.
         """
         results = OrderedDict()
         if isinstance(words, str):
             try:
-                results[words] = self.lexique[words]
+                results[words] = self.lexique[words.lower()]
             except AttributeError:
                 logger.warning('the word {} is not in Lexique383'.format(words))
         elif isinstance(words, Sequence):
             for word in words:
                 if isinstance(word, str):
                     try:
-                        results[word] = self.lexique[word]
+                        results[word] = self.lexique[word.lower()]
                     except AttributeError:
                         logger.warning('The word {} is not in Lexique383\n'.format(word))
                         continue
@@ -282,13 +321,15 @@ class Lexique383:
         Gets all lexical forms of a given word.
 
         :param word:
-        :return: List of LexItem objects sharing the same root lemma.
+            String.
+        :return:
+            List of LexItem objects sharing the same root lemma.
         """
         try:
-            lex_entry = self.lexique[word]
+            lex_entry = self.lexique[word.lower()]
         except ValueError as e:
             logger.warning('The word {} is not in Lexique383\n'.format(word))
-            raise ValueError
+            raise ValueError from e
         if isinstance(lex_entry, LexItem):
             lemmes = self.lemmes[lex_entry.lemme]
         elif isinstance(lex_entry, OrderedDict):
@@ -306,6 +347,12 @@ class Lexique383:
         """
         Saves the mismatched key/values in Lexique383 based on type coercion.
 
+        :param errors:
+            List of errors encountered while parsing Lexique38x
+        :param errors_path:
+            Path to save the errors.
+        :return:
+
         """
         with open(errors_path, 'w', encoding='utf-8') as json_file:
             json.dump(errors, json_file, indent=4)
